# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import fsolve
import time

from rk import exrk_o5, dirk_im, dirk_o4, firk_o4, firk_o6
from ode45 import ode45

# -- Globale physikalische Parameter --------------------------------------------
# Bauteile
R = 100e3  # 100 KOhm
C = 10e-9  # 10 nF
L = 50e-3  # 50 mH

# Signal
Vo = 5  # 5 V
w = 50  # 50 Hz
Vin = lambda t: Vo * np.sin(2*np.pi*w*t)

# Diodenmodell
Is = 1e-9   # 1 nA
Ut = 25e-3  # 25 mV
n = 1.0
Id = lambda t, yp: Is * (np.exp((Vin(t) - L*yp)/(n*Ut)) - 1.0)

# Parameter des ODE Lösers
n_periods = 1
n_nodes = 8000
n_steps = n_periods*n_nodes + 2*n_nodes//4 + 1
t_end = n_periods*0.02 + 2*0.005

# Gleichung
def circuit_rhs(t, y):
    dydt = np.empty_like(y)

    # TODO implementieren Sie die RHS.
    return dydt

# ------------------------------------------------------------------------
def perform_experiment(method, label):
    """ Führt das Experiment für den Löser `method` durch.

    Führt die für Aufgabe 3 d), e), f) benötigten Berechnungen für einen
    gegebenen ODE Löser durch.
    """
    t, y = run_method(method, label)

    Vout, IR, IC, ID, IL = post_process(t, y)

    plot_voltage(t, Vout, label)
    plot_current(t, IR, IC, ID, IL, label)
    plt.show()
    plt.close('all')

def run_method(method, label):
    """Berechnet die Lösung der ODE mit `method` und misst die Laufzeit.

    Input:
        method: function. ODE Löser mit Signatur wie `RungeKutta.__call__`.
         label: string. Name des Lösers (ohne Leerschläge).
    """
    t = np.zeros(n_steps)
    y = np.zeros((n_steps, 2))

    # TODO berechnet die Lösung und messt die Laufzeit.

    return t, y

def post_process(t, y):
    """Berechnet die Grössen: Vout, IR, IC, IR, ID, IL.

    Input:
        t : ndarray. Zeit.

        y : ndarray.
            Lösung der ODE.
               Achse 0: Zeit,
               Achse 1: Komponenten der Lösung.
    """
    # TODO implementieren Sie.
    Vout = np.zeros_like(t)
    IR = np.zeros_like(t)
    IC = np.zeros_like(t)
    ID = np.zeros_like(t)
    IL = np.zeros_like(t)

    return Vout, IR, IC, ID, IL

def plot_voltage(t, Vout, label):
    """Plottet die Spannung."""

    def plot():
        plt.figure()
        plt.plot(t, Vin(t), label=r'$V_{in}(t)$')
        plt.plot(t, Vout, label=r'$V_{out}(t)$')
        plt.xlabel(r'$t \; [s]$')
        plt.ylabel(r'$V \; [V]$')
        plt.ylim(-Vo - 1, Vo + 1)
        plt.grid(True)

    # Plot ohne Zoom
    plot()
    plt.legend(loc='lower right')
    plt.savefig('img/vosc_' + label + '.png')
    plt.savefig('img/vosc_' + label + '.pdf')

    # Plot mit Zoom
    plot()
    plt.xlim(0.016, 0.021)
    plt.legend(loc='upper right')
    plt.savefig('img/vosc_zoom_' + label + '.pdf')
    plt.savefig('img/vosc_zoom_' + label + '.png')

def plot_current(t, IR, IC, ID, IL, label):
    """Plottet den Stromfluss."""

    def plot():
        plt.figure()
        plt.plot(t, ID, label=r'$I_D(t)$')
        plt.plot(t, IR, label=r'$I_R(t)$')
        plt.plot(t, IC, label=r'$I_C(t)$')
        plt.plot(t, IL, label=r'$I_L(t)$')
        plt.grid(True)
        plt.xlabel(r'$t \; [s]$')
        plt.ylabel(r'$I \; [A]$')

    # Plot ohne Zoom
    plot()
    plt.legend(loc='upper right')
    plt.savefig('img/iosc_' + label + '.pdf')

    # Plot mit Zoom
    plot()
    plt.xlim(0.016, 0.021)
    plt.ylim(-0.0025, 0.0025)
    plt.legend(loc='upper right')
    plt.savefig('img/iosc_zoom_' + label + '.pdf')

def ode45_interface(rhs, ic, t_end, n_steps):
    return ode45(rhs, [0.0, t_end], ic,
                 stat="on", reltol=1e-8, abstol=1e-8, initialstep=2e-5)

if __name__ == '__main__':
    perform_experiment(exrk_o5(), "exrk_o5")
    perform_experiment(dirk_im(), "dirk_im")
    perform_experiment(dirk_o4(), "dirk_o4")
    perform_experiment(firk_o4(), "firk_o4")
    perform_experiment(firk_o6(), "firk_o6")
    perform_experiment(ode45_interface, "ode45")
